# Time-stamp: <2012-04-20 12:56:49 chl>

#
# Overview of GEE modeling.
#

save.fig <- TRUE

library(geepack)

data(ohio)
head(ohio, n=8)
length(unique(ohio$id))

# Look at the individual pattern of responses
smoke.yes <- xtabs(resp ~ id + age, subset(ohio, smoke==1))
smoke.no <- xtabs(resp ~ id + age, subset(ohio, smoke==0))

# Show the marginal distribution (mean across 4 time points)
marg.means <- data.frame(resp=c(apply(smoke.yes, 2, mean), apply(smoke.no, 2, mean)),
                         age=gl(4, 1, labels=7:10, ordered=TRUE),
                         smoke=gl(2, 4, labels=c("smoking", "not smoking")))
library(lattice)
library(latticeExtra)

# Note: type=c("a","g") is simpler
plt1 <- xyplot(resp ~ age, data=marg.means, group=smoke, type=c("l","g"),
               xlab="Age (years)", ylab="Wheeziness (%)", lwd=2)

if (save.fig) pdf("fig-ohio.pdf", width=8, height=4)
update(plt1, par.settings = custom.theme())
if (save.fig) dev.off()

# Try a base model with exchangeable correlation structure
fm <- resp ~ age*smoke
gee.fit.exch <- geese(fm, id=id, data=ohio, family=binomial, corstr="exch",
                      scale.fix=TRUE)
# Compute the robust SEs
print(rob.se <- sqrt(diag(gee.fit.exch$vbeta)))

# or
gee.fit.exch <- geeglm(fm, id=id, data=ohio, family=binomial, corstr="exch",
                       scale.fix=TRUE, std.err="san.se")
summary(gee.fit.exch)

# in the latter case, robust SEs are found to be in
print(rob.se <- sqrt(diag(gee.fit.exch$geese$vbeta)))

# We can test whether `smoke` is significant using a Wald test
gee.fit.exch2 <- geeglm(update(fm, . ~ . - age), id=id, data=ohio, family=binomial,
                        corstr="exch", scale.fix=TRUE, std.err="san.se")
anova(gee.fit.exch, gee.fit.exch2)

# Compute 95% CIs for time (odds-ratio)
exp(coef(gee.fit.exch)["age"] + c(-1,1)*rob.se[2] * qnorm(0.975))

# Or, we can compute any odds-ratio with 95% CIs as follows
library(doBy)
gee.fit.ci <- esticon(gee.fit.exch, diag(4))
exp(gee.fit.ci[,c("Estimate","Lower","Upper")])

# Compare to a model with unstructured correlation structure,
# and see what is going during estimation
gee.fit.unst <- geeglm(fm, id=id, data=ohio, family=binomial,
                       corstr="uns", scale.fix=TRUE, std.err="san.se",
                       control=geese.control(trace=TRUE))
summary(gee.fit.unst)

# And finally to a model assuming independence between measurements,
# with model-based SEs
library(gee)
gee.fit.inde <- gee(fm, id=id, data=ohio, family=binomial,
                    corstr="independence", scale.fix=TRUE)
summary(gee.fit.inde)

# Remove the interaction term and compute predictions (age treated as continuous)
gee.fit.exch3 <- geeglm(update(fm, . ~ . - age:smoke), id=id, data=ohio, family=binomial,
                        corstr="exch", scale.fix=TRUE, std.err="san.se")
newdf <- expand.grid(age=seq(-2, 1, length=50), smoke=c(0,1))
newdf <- cbind.data.frame(resp=predict(gee.fit.exch3, newdata=newdf), newdf)
                     
plt2 <- xyplot(exp(resp) ~ age, data=newdf, group=smoke, type=c("l","g"),
               xlab="Age (years)", ylab="Wheeziness (%)",
               auto.key=list(corner=c(0,0), text=c("No","Yes"), cex=.8))
update(plt2, par.settings = custom.theme())

### Experimental -- don't use!
# same with age treated as categorical
fm2 <- resp ~ age + factor(smoke, labels=7:10)
fm3 <- resp ~ age * factor(smoke, labels=7:10)

gee.fit3 <- geeglm(fm2, id=id, data=ohio, family=binomial, corstr="exch", scale.fix=TRUE)
gee.fit4 <- geeglm(fm3, id=id, data=ohio, family=binomial, corstr="exch", scale.fix=TRUE)

ohio$fit3 <- exp(predict(gee.fit3))
ohio$fit4 <- exp(predict(gee.fit4))

xyplot(resp ~ as.factor(age), data=ohio, group=smoke, type=c("a","g"))

xyplot(fit4 ~ factor(age, labels=7:10), data=ohio, group=smoke, type=c("a","g"))
### End (experimental)

# GLMM
library(lme4)

fit.glmm <- lmer(resp ~ age+smoke+(1|id), data=ohio, family=binomial)

head(fitted(fit.glmm))

# Show observed vs. fitted responses
library(languageR)
plotlogistic.fit.fnc(fit.glmm, data=ohio)

# Use MCMC
set.seed(101)
fit.glmm.mcmc <- mcmcsamp(fit.glmm, 500)


# GLMM vs. GEE predicted probabilities
# Code from http://www.unc.edu/courses/2010spring/ecol/562/001/docs/lectures/lecture14.htm
gee.fitc <- geeglm(resp ~ as.numeric(age) + smoke, id=id, 
                   data=ohio, family=binomial, 
                   corstr="exch", scale.fix=TRUE)
glmm.fitc <- lmer(resp ~ as.numeric(age) + smoke + (1|id), data=ohio,
                  family=binomial)

exp(cbind(coef(gee.fitc), fixef(glmm.fitc)))

gee.pr <- function(x) {
  eta <- coef(gee.fitc)[1] + coef(gee.fitc)[2]*x + coef(gee.fitc)[3]
  return(exp(eta) / (1+exp(eta)))
}

glmm.pr <- function(x, u=0) {
  eta <- fixef(glmm.fitc)[1] + fixef(glmm.fitc)[2]*x + fixef(glmm.fitc)[3] + u
  return(exp(eta) / (1+exp(eta)))
}


curve(gee.pr, from=-2, to=1, ylim=c(0, 1), ylab="Probability of wheeziness",
      xlab="Age (yrs.)", axes=FALSE)
curve(glmm.pr(x, 0), add=T, col=1, lwd=2)

sapply(unique(ranef(glmm.fitc))[[1]][,1], function(y) curve(glmm.pr(x,y), add=T, col='grey70', lty=2))

axis(1, at=seq(-2, 1, by=.5), labels=seq(7, 10, by=.5))
axis(2)
box()


library(plyr)

mean.res <- ddply(ohio, ~ smoke + factor(age),
                  summarise,
                  freq=mean(resp), gee=mean(fit3), glm=)



library(orth)
data(six.city)
z.6city <- #
orth.fit <- orth(resp ~ age + smoke, data=ohio, formula.z=~ age + smoke,
                 id=id, dataz=ohio)


library(yags)
yags.fit <- yags(resp ~ age + smoke, data=ohio, id=id, family=binomial,
                 corstr="exchangeable", alphainit=0)
exp(yags.fit@coefficients)

library(alr)
alr.fit <- alr(resp ~ age + smoke, id=id, data=ohio, depm="exchangeable", ainit=0.01)
summary(alr.fit)
